
// Buttons.java
// Andrew Davison, June 2014, ad@fivedots.coe.psu.ac.th

/* Report button activity on the wiimote:
    pressed, released, held

    Held tests only return true for a button when another
    button is pressed after the 'held' one.


    Also demo the rumble and LED lights.
      - rumble on/off  using +/- buttons
      - LEDs lights changed using arrow keys

    The 'A' button causes a disconnect.
*/

import wiiusej.*;
import wiiusej.wiiusejevents.physicalevents.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;
import wiiusej.wiiusejevents.wiiuseapievents.*;


public class Buttons implements WiimoteListener
{
  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }

  private Wiimote wiimote;
  private boolean showStatus = true;
  private int count = 0;


  public Buttons() 
  {
    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);  // no rumble on connection
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      return;
    }
    else 
      System.out.println("No. of wiimotes found: " + wiimotes.length);

    wiimote = wiimotes[0];
    wiimote.setTimeout((short)20, (short)20);     // default of 10 causes packet timeouts

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();
  }  // end of Buttons()



  public void onButtonsEvent(WiimoteButtonsEvent e) 
  {
    if (haveButtonsChanged(e)) {
      // System.out.println("\nButton: " + e);
      buttonAction(e);
      lightsAction(e);
      rumbleAction(e);
    }

    // say goodbye using "A"
    if (e.isButtonAPressed()) {
      WiiUseApiManager.shutdown();
      System.out.println("Wiimote shutdown");
      System.exit(1);
    }
  }  // end of onButtonsEvent()


  private boolean haveButtonsChanged(ButtonsEvent e)
  {
    return ((e.getButtonsJustPressed() != 0) ||
            (e.getButtonsJustReleased() != 0) ||
            (e.getButtonsHeld() != 0) );
  }



  private void buttonAction(WiimoteButtonsEvent e)
  /* held tests returns true for a button only if another 
     event arrives, such as a motion even, after the button
     has been pressed

     There is also a isButtonXXXPressed() which combines 
     "JustPressed" and "Held"; it means that the button is
     currently pressed.
  */
  {
    if (e.isButtonOneJustPressed())
      System.out.println("One pressed");
    if (e.isButtonOneHeld())
      System.out.println("+ One held");
    if (e.isButtonOneJustReleased())
      System.out.println("  One released");

    if (e.isButtonTwoJustPressed())
      System.out.println("Two pressed");
    if (e.isButtonTwoHeld())
      System.out.println("+ Two held");
    if (e.isButtonTwoJustReleased())
      System.out.println("  Two released");

    if (e.isButtonAJustPressed())
      System.out.println("A pressed");
    if (e.isButtonAHeld())
      System.out.println("+ A held");
    if (e.isButtonAJustReleased())
      System.out.println("  A released");

    if (e.isButtonBJustPressed())
      System.out.println("B pressed");
    if (e.isButtonBHeld())
      System.out.println("+ B held");
    if (e.isButtonBJustReleased())
      System.out.println("  B released");

    if (e.isButtonLeftJustPressed())
      System.out.println("Left pressed");
    if (e.isButtonLeftHeld())
      System.out.println("+ Left held");
    if (e.isButtonLeftJustReleased())
      System.out.println("  Left released");

    if (e.isButtonRightJustPressed())
      System.out.println("Right pressed");
    if (e.isButtonRightHeld())
      System.out.println("+ Right held");
    if (e.isButtonRightJustReleased())
      System.out.println("  Right released");

    if (e.isButtonUpJustPressed())
      System.out.println("Up pressed");
    if (e.isButtonUpHeld())
      System.out.println("+ Up held");
    if (e.isButtonUpJustReleased())
      System.out.println("  Up released");

    if (e.isButtonDownJustPressed())
      System.out.println("Down pressed");
    if (e.isButtonDownHeld())
      System.out.println("+ Down held");
    if (e.isButtonDownJustReleased())
      System.out.println("  Down released");

    if (e.isButtonMinusJustPressed())
      System.out.println("Minus pressed");
    if (e.isButtonMinusHeld())
      System.out.println("+ Minus held");
    if (e.isButtonMinusJustReleased())
      System.out.println("  Minus released");

    if (e.isButtonPlusJustPressed())
      System.out.println("Plus pressed");
    if (e.isButtonPlusHeld())
      System.out.println("+ Plus held");
    if (e.isButtonPlusJustReleased())
      System.out.println("  Plus released");

    if (e.isButtonHomeJustPressed())
      System.out.println("Home pressed");
    if (e.isButtonHomeHeld())
      System.out.println("+ Home held");
    if (e.isButtonHomeJustReleased())
      System.out.println("  Home released");
  }  // end of buttonAction()




  private void lightsAction(WiimoteButtonsEvent e)
  // use the arrow keys to affect the LEDs
  {
    if (e.isButtonLeftJustPressed())
      wiimote.setLeds(true, false, false, false);

    if (e.isButtonRightJustPressed())
      wiimote.setLeds(false, false, false, true);

    if (e.isButtonUpJustPressed())
      wiimote.setLeds(false, true, false, false);

    if (e.isButtonDownJustPressed())
      wiimote.setLeds(false, false, true, false);

  }  // end of lightsAction()



  private void rumbleAction(WiimoteButtonsEvent e)
  // use the +/- buttons to switch rumble on/off
  {
    if (e.isButtonPlusJustPressed()) {
      System.out.println("Rumble activated");
      wiimote.activateRumble();
    }

    if (e.isButtonMinusJustPressed()) {
      System.out.println("Rumble deactivated");
      wiimote.deactivateRumble();
         // has no effect if arrow key pressed beforehand;
         // press any arrow key to get the rumble to switch off
    }
  }  // end of rumbleAction()




  // ---------------------- other wiimote event processing ---------------------------


  public void onStatusEvent(StatusEvent e) 
  { if (showStatus) {
      System.out.println("\n" + e);
      showStatus = false;
    }
  }

  public void onDisconnectionEvent(DisconnectionEvent e)
  {  System.out.println("\nDisconnection): " + e);   
     // not activated when wiimote unplugged 
  }

  public void onIrEvent(IREvent e) {}

  public void onMotionSensingEvent(MotionSensingEvent e)
  {  System.out.println("Motion sensing(" + (count++) + "): " + e);  }


  public void onExpansionEvent(ExpansionEvent e){}  

  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e) {}
  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e) {}

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e) {}
  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e) {}

  public void onNunchukInsertedEvent(NunchukInsertedEvent e) {}
  public void onNunchukRemovedEvent(NunchukRemovedEvent e) {}


  // ---------------------------------------------------------

  public static void main(String[] args)
  {  new Buttons();  }  

}  // end of Buttons class

