
// IRTracker.java
// Andrew Davison, June 2014, ad@fivedots.coe.psu.ac.th

/* Demonstrate the use of the wiimote's IR tracking using a sensor bar.

   Report IR settings at start-up time, and then the wiimote's
   IR tracking position on the user's virtual screen (the "Track" print-out). 

   If reportPts() is called then the underlying detected IR points are
   also printed out. The wiimote can detect up to 4 IR sources. 
*/

import java.io.*;

import wiiusej.*;
import wiiusej.values.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;
import wiiusej.wiiusejevents.wiiuseapievents.*;
import wiiusej.wiiusejevents.physicalevents.*;



public class IRTracker implements WiimoteListener
{

  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }



  private Wiimote wiimote;
  private boolean showStatus = true;
  private boolean showIRSettings = true;   // shown only at start-up


  public IRTracker() 
  {
    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);  
                                       // do not rumble on connection
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      return;
    }
    else 
      System.out.println("No. of wiimotes found: " + wiimotes.length);

    wiimote = wiimotes[0];
    wiimote.setTimeout((short)20, (short)20);     // default of 10 causes packet timeouts

    wiimote.activateIRTRacking();
    wiimote.activateMotionSensing();   // IMPORTANT: improves the accuracy of IR

    wiimote.setIrSensitivity(5);       // maximum; ranges between 1-5; default is 3

    wiimote.setSensorBarAboveScreen();
    wiimote.setScreenAspectRatio169(); 
                // does not seem to work; aspect ratio still 4/3 in status report

    wiimote.setVirtualResolution(1920, 1080);   // desktop machine
    // wiimote.setVirtualResolution(1366, 768);      // laptop
         // do this last, after screen and aspect ratio change

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();
  }  // end of IRTracker()



  // ------------------ event handlers -------------------

  public void onIrEvent(IREvent e)
  {  
    if (showIRSettings) {
      reportIRSettings(e);    // show only at start-up
      showIRSettings = false;
    }

   if ((e.getX() != 0) || (e.getY() != 0)) {       // don't print (0,0,??) to reduce the output
      System.out.printf("Track: (%d, %d, %.0f);  \t", e.getX(), e.getY(), e.getZ());
          /* This is the current IR tracking position on 
             the user-specified virtual screen.
             The x- and y-values can be negative, and exceed the maximums
             defined by the virtual screen size. 
             The z- float value can be very inaccurate; it represents
             the distance from the screen. 
          */

      System.out.print("Abs: (" + e.getAx() + ", " + e.getAy() +")");
                    // absolute position; can still be negative;
                    // there's nothing equivalent to this in the wiiuse library 
      System.out.println();

      IRSource[] irPts = e.getIRPoints(); 
           // get the points used to calculate the tracking position;
           // A wiimote can see up to 4 IR sources, but this method may return null or 0 pts
      reportPts(irPts);
    }
  }  // end of onIrEvent()





  private void reportIRSettings(IREvent e)
  {
    System.out.println("============ IR Settings ================");
    if (e.isScreenAspectRatio169())
      System.out.println("Screen aspect ratio: 16/9");
    else if (e.isScreenAspectRatio43())
      System.out.println("Screen aspect ratio: 4/3");
    
    System.out.println("Virtual screen res: " + e.getXVRes() + "x" + e.getYVRes());

    // is IR sensor bar centered above/below the TV/monitor?
    if (e.isSensorBarAbove())
      System.out.println("Sensor bar position: above");
    else if (e.isSensorBarBelow())
      System.out.println("Sensor bar position: below");

    System.out.println("IR (X,Y) correction offsets: (" + e.getXOffset() + ", " +
                                                          e.getYOffset() + ")");
    System.out.println("IR camera sensitivity: " + e.getIrSensitivity());
           // sensitivity is between 1 (lowest) and 5 (highest). The default is 3

    System.out.println();
  }  // end of reportIRSettings()



  private void reportPts(IRSource[] irPts)
  /* Both the raw and corrected coords use a fixed virtual screen 
     resolution of 1024 x 768, with (0,0) at top-left. The corrected
     values are smoothed versions of the raw data, and are used to
     calculate the single IR tracking location.
     The size can range from 1-4; 4 is bigger, so the dot is closer to the IR camera.
  */
  {
    if ((irPts != null) && (irPts.length > 0)) {
      for(IRSource ir : irPts) {
        System.out.println( "  Corr: (" + ir.getX() + "," + ir.getY() +
                           ");  Raw: (" + ir.getRx() + "," + ir.getRy() +
                            "); Size: " + ir.getSize());
      }
    }
    System.out.println();
  }  // end of reportPts()



  public void onButtonsEvent(WiimoteButtonsEvent e) 
  {
    // say goodbye using "A"
    if (e.isButtonAPressed()) {
      WiiUseApiManager.shutdown();
      System.out.println("Wiimote shutdown");
      System.exit(0);
    }
  }  // end of onButtonsEvent()



  public void onStatusEvent(StatusEvent e)
  { if (showStatus) {
      System.out.println("\n" + e);
      showStatus = false;
    }
  }


  public void onDisconnectionEvent(DisconnectionEvent e)  {}

  public void onMotionSensingEvent(MotionSensingEvent e){ }

  public void onExpansionEvent(ExpansionEvent e)  { }

  public void onNunchukInsertedEvent(NunchukInsertedEvent e) { }
  public void onNunchukRemovedEvent(NunchukRemovedEvent e) { }

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e)  { }
  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e) { }

  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e) { }
  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e) { }


  // ---------------------------------------------------------


  public static void main(String[] args)
  {   new IRTracker(); } 

}  // end of IRTracker class

