
// Motion.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2014

/*  Demonstrates the use of the wiimote's motion sensing, by
    reporting orientation and g-forces along the 3 axes.
*/

import wiiusej.*;
import wiiusej.values.*;
import wiiusej.wiiusejevents.physicalevents.*;
import wiiusej.wiiusejevents.wiiuseapievents.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;


public class Motion implements WiimoteListener
{

  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }

  private int eventCount = 0;
  private boolean showStatus = true;
  private boolean showMotionSettings = true;



  public Motion() 
  {
    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      return;
    }
    else 
      System.out.println("No. of wiimotes found: " + wiimotes.length);


    Wiimote wiimote = wiimotes[0];
    wiimote.setTimeout((short)20, (short)20);     // default of 10 causes packet timeouts

    wiimote.activateMotionSensing(); 
       // onMotionSensingEvent() will be called, repeatedly

    // wiimote.activateIRTRacking();   // needed to generate yaw information


    wiimote.setOrientationThreshold(3.0f);    // was 0.5 degrees
    wiimote.setAccelerationThreshold(10);      // default is 5
    wiimote.activateSmoothing();
    wiimote.setAlphaSmoothingValue(0.2f);        
                   // Accelerometer smoothing [0-1]; was 0.07
      /* Accelerometers produce a lot of noise, so to reduce this wiiuse 
         has implemented an exponential moving average for each angle.
         setAlphaSmoothingValue() changes the alpha value of the equation.
      */

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();
  }  // end of Motion()




  public void onButtonsEvent(WiimoteButtonsEvent e)
  { 
    // say goodbye using "A"
    if (e.isButtonAPressed()) {
      WiiUseApiManager.shutdown();
      System.out.println("Wiimote shutdown");
      System.exit(1);
    }
  }


  public void onMotionSensingEvent(MotionSensingEvent e)
  /* Motion sensing can return information on the orientation,
     accelerations in terms of g-forces, or raw accelerations. 
     Uncomment the prints to report orientation, g-forces,
     and raw acceleration, but the output is large.
  */
  {  
    if (showMotionSettings) {    // report only at start-up
      reportMotionSettings(e);
      showMotionSettings = false;
    }

    Orientation ori = e.getOrientation();
    // System.out.printf("%d. Pitch(x): %.1f  Roll(y): %.1f  Yaw(z): %.1f\n", 
    //       eventCount++, ori.getPitch(), ori.getRoll(), ori.getYaw());  
        /* roll and pitch range is -180 to 180 degrees;               
           yaw is -26 to 26 degrees (only works if IR sensing is activated)
             roll: around y-axis
             pitch: around x-axis
             yaw: around z-axis
        */

    GForce gforce = e.getGforce();    // in range +/- 3 gravity units
    // System.out.printf("       %d. Gravity z: %.2f\n", eventCount++, gforce.getZ());  
    //System.out.printf("       %d. Gravity x: %.2f  y: %.2f  z: %.2f\n", 
    //             eventCount++, gforce.getX(), gforce.getY(), gforce.getZ());  
      
    reportFlicks(gforce);

    RawAcceleration ra = e.getRawAcceleration();
    // System.out.println("  " + ra);
            // zero raw acceleration is reported as 0x80, 128
  }  // end of onMotionSensingEvent()




  private void reportMotionSettings(MotionSensingEvent e)
  {
    System.out.println("============ Motion Sensing Settings ================");
    System.out.println("Orientation threshold: " + e.getOrientationThreshold());
    System.out.println("Acceleration threshold: " + e.getAccelerationThreshold());

    if (e.isSmoothingActive())
      System.out.println("Alpha smoothing: " + e.getAlphaSmoothing());
    else 
      System.out.println("Smoothing not active");
    System.out.println();
  }  // end of reportMotionSettings()



  private void reportFlicks(GForce gf)
  // convert g-forces into movement directions
  {
    float xForce = gf.getX();
    if (xForce > 1.5f)
      System.out.println("Right");
    else if (xForce < -1.5f)
      System.out.println("Left");

    float yForce = gf.getY();
    if (yForce > 1.5f)
      System.out.println("  Forward");
    else if (yForce < -1.5f)
      System.out.println("  Back");

    float zForce = gf.getZ();
    if (zForce > 1.8f)
      System.out.println("    Down");
    else if (zForce < -1.0f)
      System.out.println("    Up");
  }  // end of reportFlicks()



  public void onStatusEvent(StatusEvent e)
  { if (showStatus) {
      System.out.println("\n" + e);
      showStatus = false;
    }
  }


  public void onIrEvent(IREvent e){}


  public void onDisconnectionEvent(DisconnectionEvent e) {}

  public void onExpansionEvent(ExpansionEvent e) {}  

  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e) {}
  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e) {}

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e) {}
  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e) {}

  public void onNunchukInsertedEvent(NunchukInsertedEvent e) {}
  public void onNunchukRemovedEvent(NunchukRemovedEvent e) {}


  // ---------------------------------------------------


  public static void main(String[] args)
  {  new Motion();  }  

}  // end of Motion class
