
// Nunchuk.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2014

/*  Monitor the Nunchuk attachment to the wiimote
       - print the state of the C and Z buttons
       - print the rotation and magnitude of the joystick
       - print its motion sensing info (orientation, accelerometer)
*/

import wiiusej.*;
import wiiusej.values.*;
import wiiusej.wiiusejevents.physicalevents.*;
import wiiusej.wiiusejevents.wiiuseapievents.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;


public class Nunchuk implements WiimoteListener
{

  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }

  private boolean showStatus = true;
  private int eventCount = 0;


  public Nunchuk() 
  {
    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);  // do not rumble on connection
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      return;
    }
    else 
      System.out.println("No. of wiimotes found: " + wiimotes.length);

    Wiimote wiimote = wiimotes[0];

    wiimote.setTimeout((short)20, (short)20);     // default of 10 causes packet timeouts


    // wiimote.activateMotionSensing();   // not needed for Nunchuk motion sensing

    // for wiimote
    wiimote.setOrientationThreshold(3.0f);    // was 0.5 degrees
    wiimote.setAccelerationThreshold(10);     // default is 5

    // for Nunchuk
    wiimote.setNunchukOrientationThreshold(3.0f);    // was 0.5 degrees
    wiimote.setNunchukAccelerationThreshold(10);     // default is 5

    // for wiimote *and* nunchuk
    wiimote.activateSmoothing();
    wiimote.setAlphaSmoothingValue(0.2f);     // was 0.07
      // exponential moving average for the acceloremeter angles 

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();
  }  // end of Nunchuk()




  public void onButtonsEvent(WiimoteButtonsEvent e)
  { 
    // say goodbye using "A"
    if (e.isButtonAPressed()) {
      WiiUseApiManager.shutdown();
      System.out.println("Wiimote shutdown");
      System.exit(1);
    }
  }



  public void onIrEvent(IREvent e) {}

  public void onMotionSensingEvent(MotionSensingEvent e) 
  { System.out.println("Motion Sensing: " + e);  }   // not used



  public void onStatusEvent(StatusEvent e)
  { if (showStatus) {
      System.out.println("\n" + e);
      showStatus = false;
    }
  }


  public void onDisconnectionEvent(DisconnectionEvent e)
  {  System.out.println("Disconnection: " + e);  }



  public void onExpansionEvent(ExpansionEvent e)
  // cast expansion event to a Nunchuk event, and process it
  {
    if (e instanceof NunchukEvent)
      processNunchuk((NunchukEvent) e);
    else
      System.out.println("Unknown Expansion Event: " + e); 
  }  // end of onExpansionEvent()



  private void processNunchuk(NunchukEvent ne)
  /*   - print the rotation and magnitude of the joystick
       - print the state of the C and Z buttons
       - print its motion sensing info (orientation, accelerometer)
  */
  {
    // System.out.println("Nunchuk Event: " + ne); 
         // shows all button, joystick, motion sensing info 

    if (ne.isThereNunchukJoystickEvent()) {
      JoystickEvent je = ne.getNunchukJoystickEvent();
      float magnitude = je.getMagnitude();
      if (magnitude > 0.1)
        System.out.printf("Joystick angle %.1f; magnitude: %.2f\n",  
                                  je.getAngle(), magnitude);
    }

    NunchukButtonsEvent nbe = ne.getButtonsEvent();
    if (haveButtonsChanged(nbe))
      nunchukButtons(nbe);

    if (ne.isThereMotionSensingEvent()) {
      MotionSensingEvent nme =  ne.getNunchukMotionSensingEvent();

      Orientation ori = nme.getOrientation();
      System.out.printf("%d. Pitch(x): %.1f  Roll(y): %.1f  Yaw(z): %.1f\n", 
             eventCount++, ori.getPitch(), ori.getRoll(), ori.getYaw());  
         // yaw is always 0 for the nunchuk

      GForce gforce = nme.getGforce();
      // System.out.printf("%d. Gravity z: %.2f\n", eventCount++, gforce.getZ());  
      // System.out.printf("%d. Gravity x: %.2f  y: %.2f  z: %.2f\n", 
      //              eventCount++, gforce.getX(), gforce.getY(), gforce.getZ());  

      RawAcceleration ra = nme.getRawAcceleration();
      // System.out.println("  " + ra);
    }
  }  // end of processNunchuk()



  private boolean haveButtonsChanged(ButtonsEvent e)
  {
    return ((e.getButtonsJustPressed() != 0) ||
            (e.getButtonsJustReleased() != 0) ||
            (e.getButtonsHeld() != 0) );     // held test always returns 0
  }



  private void nunchukButtons(NunchukButtonsEvent e)
  /* held tests work for the Nunchuk since it is sending motion info
     all the time unlike the wiimote

     There is also a isButtonXXXPressed() which combines 
     "JustPressed" and "Held"; it means that the button is
     currently pressed.

     'C' is above 'Z' at the front of the Nunchuk
  */
  {
    if (e.isButtonCJustPressed())
      System.out.println("C pressed");
    if (e.isButtonCHeld())
      System.out.println("+ C held");
    if (e.isButtonCJustReleased())
      System.out.println("  C released");

    if (e.isButtonZJustPressed())
      System.out.println("Z pressed");
    if (e.isButtonZeHeld())    // note spelling mistake
      System.out.println("+ Z held");
    if (e.isButtonZJustReleased())
      System.out.println("  Z released");
  }  // end of nunchukButtons()


  public void onNunchukInsertedEvent(NunchukInsertedEvent e)
  { System.out.println("\nNunchuk Inserted: " + e); }

  public void onNunchukRemovedEvent(NunchukRemovedEvent e)
  {  System.out.println("\nNunchuk Removed: " + e); }



  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e)
  {  System.out.println(e); }

  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e)
  {  System.out.println(e); }

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e)
  {  System.out.println(e); }

  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e)
  {  System.out.println(e); }

  // ---------------------------------------------------


  public static void main(String[] args)
  {  new Nunchuk();  }  

}  // end of Nunchuk class
