
// WiiPosPanel.java
// Andrew Davison, June 2014, ad@fivedots.coe.psu.ac.th

/* Creates a drawing surface which covers the entire screen.

   The run() method enters a  3-element loop:
      update by reading the wiimote  / draw the panel / sleep

   The sleep time tries to keep each cycle close to DELAY ms.

   The app ends when:
     * the user presses ESC, q, ctrl-c on the keyboard
    or
     * the user presses the "A" button on the wiimote
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;



public class WiiPosPanel extends JPanel implements Runnable
{
  private static final int DELAY = 25;  // ms  (redraw interval)

  private boolean isRunning = false;   // used to stop the loop
  private WiiReader wiimote;



  public WiiPosPanel()
  {
    // set panel/game dimensions to be those of the entire screen
    Dimension scrDim = Toolkit.getDefaultToolkit().getScreenSize();   // screen size
    setPreferredSize(scrDim); 

    int pWidth = scrDim.width;
    int pHeight = scrDim.height;
    System.out.println("Panel (w,h) : (" + pWidth + ", " + pHeight + ")");

    // listen for key presses
    setFocusable(true);
    requestFocus();    // the JPanel now has focus, so receives key events
    initKeyListener();

    wiimote = new WiiReader(pWidth, pHeight);  // start the wiimote

    new Thread(this).start();   // start updating the panel
  } // end of WiiPosPanel()



  private void initKeyListener()
  // define keys for stopping
  {
    addKeyListener( new KeyAdapter() {
       public void keyPressed(KeyEvent e)
       { 
         int keyCode = e.getKeyCode();
         if ((keyCode == KeyEvent.VK_ESCAPE) || (keyCode == KeyEvent.VK_Q) ||
             ((keyCode == KeyEvent.VK_C) && e.isControlDown()) )
                // ESC, q, ctrl-c to stop isRunning 
           isRunning = false;
       }
     });
  }  // end of initKeyListener()




  public void run()
  /* Implements the app loop: update / draw / sleep
     The sleep time tries to keep each cycle close to DELAY ms.
  */
  {
    long duration;
    isRunning = true;
    while(isRunning && !wiimote.wantsToFinish()) {
      long startTime = System.currentTimeMillis();
      update();
      duration = System.currentTimeMillis() - startTime;
      // System.out.println(" update duration: " + duration);
      repaint();

      if (duration < DELAY) {
        try {
          Thread.sleep(DELAY-duration);  // wait until DELAY time has passed
        } 
        catch (Exception ex) {}
      }
    }

    wiimote.closeDown();
    System.exit(0);
  }  // end of run()


  private void update()
  /* read the wiimote's data, and do something */
  {  
    // System.out.println( wiimote.getRoll());
  }  // end of update()


  public void closeDown() 
  {  isRunning = false; } 



  // --------------------- rendering ------------------------------


  public void paintComponent(Graphics g)
  /* draw the current state of the application, and let WiiReader draw
     any wiimote graphics */
  { 
    super.paintComponent(g);
    Graphics2D g2d = (Graphics2D)g; 

    // use antialiasing
    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                          RenderingHints.VALUE_ANTIALIAS_ON);

    wiimote.draw(g2d); 
  } // end of paintComponent()


} // end of WiiPosPanel class
