
// WiiReader.java
// Andrew Davison, June 2014, ad@fivedots.coe.psu.ac.th

/* Sets up the wiimote, turning on IR and motion sensing.

   Calculates a IR tracking location for the wiimote, based on
   the detected IR points.

   Displays information on the line width between the IR points,
   rotation, and wiimote view boundary ranges
   for detection of the sensor bar.
*/

import java.awt.*;
import java.io.*;
import java.awt.image.*;

import wiiusej.*;
import wiiusej.values.*;
import wiiusej.wiiusejevents.physicalevents.*;
import wiiusej.wiiusejevents.wiiuseapievents.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;



public class WiiReader implements WiimoteListener
{
  private static final double POINT_PROXIMITY = 80;
            // pixel distance between two points which is deemed 'close'

  private static final int DOT_SIZE = 15;
            // used for drawing points in the panel


  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }

  private int pWidth, pHeight;   // panel dimensions

  private Wiimote wiimote;

  private boolean wantsToFinish = false;
  private boolean showStatus = true;
  private Font msgsFont;


  // IR data
  private IRSource[] irPts;   // detected IR points

  private Point leftIR, rightIR;
      /* two IR points that are different from each other, and so 
         are assumed to be the IR sources at the
         left and right ends of the sensor bar */

  private int widthIR;   // distance between leftIR and rightIR points
           /* this can be treated as the line width of the current view
              of the sensor bar, assuming the two points are light sources
              at either end of the bar */

  private Point currLoc;    // current IR tracking point for the wiimote


  private int xMinView, xMaxView, yMinView, yMaxView;
     // wiimote's view boundaries for the sensor bar

  private int rollAngle = 0;    // roll detected for wiimote



  public WiiReader(int w, int h)
  {
    pWidth = w; pHeight = h;

    // start by drawing the IR tracking point at the center of the panel
    currLoc = new Point(pWidth/2, pHeight/2);

    // guess at starting locations for IR points and their distance apart
    leftIR = new Point((pWidth/2 - 50), pHeight/2);
    rightIR = new Point((pWidth/2 + 50), pHeight/2);
    widthIR =  (int) Math.round(leftIR.distance(rightIR));

    // set the initial view range to be the same as the panel dimensions
    xMinView = 0; yMinView = 0;
    xMaxView = pWidth-1; yMaxView = pHeight-1;

    msgsFont = new Font("SansSerif", Font.BOLD, 24);

    wiimote = initWiimote(pWidth, pHeight);
  }  // end of WiiReader()




  private Wiimote initWiimote(int pWidth, int pHeight)
  {
    System.out.println("Initializing wiimote");

    if (WiiUseApiManager.getNbConnectedWiimotes() == 0) {
      System.out.println("No wiimotes connected");
      wantsToFinish = true;
      return null;
    }

    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      wantsToFinish = true;
      return null;
    }
    Wiimote wiimote = wiimotes[0];
    wiimote.setTimeout((short)20, (short)20);     // default of 10 causes packet timeouts

    wiimote.activateIRTRacking();
    wiimote.activateMotionSensing();  // IMPORTANT: improves the accuracy of IR

    wiimote.setIrSensitivity(5);   // maximum; ranges between 1-5

    wiimote.setSensorBarAboveScreen();
      // don't both with virtual screen settings since only using detected IR points

    wiimote.setOrientationThreshold(1);           // was 0.5 degrees
          /* the amount an angle (roll, pitch, or yaw) must change 
             before generating an event */

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();
    return wiimote;
  }  // end of initWiimote()



  // ---------------------- IR sensing ---------------------------

  public void onIrEvent(IREvent e)
  {  
    irPts = e.getIRPoints(); 
    // reportPts(irPts);
    Point loc = locateIR(irPts);
    if (loc != null) {
      // System.out.println("  Current Loc: (" + loc.x + ", " + loc.y + ")");
      currLoc = loc;
    }
  }  // end of onIrEvent()




  private void reportPts(IRSource[] irPts)
  /* Print the corrected IR points which use a fixed virtual screen 
     resolution of 1024 x 768, with (0,0) at top-left. The corrected
     values are smoothed versions of the raw data.
  */
  {
    if ((irPts == null) || (irPts.length == 0))
      return;
    for(IRSource ir : irPts)
      System.out.print( "(" + ir.getX() + "," + ir.getY() + ")  ");
    System.out.println();
  }  // end of reportPts()





  private Point locateIR(IRSource[] irPts)
  /* Choose two IR points from the current detected IR points
     to represent the IR sources at the
     two ends of the sensor bar (leftIR, rightIR). If there aren't two
     suitable points, then use one point and estimate the other based on
     the previous values of the IR points.

     Use leftIR and rightIR to calculate a current
     IR tracking location and the line width of the bar.

     Parts of this algorithm were inspired by a discussion of sensor bar
     point calculation at http://wiibrew.org/wiki/Wiimote/Pointing
 */
  {
    if ((irPts != null) && (irPts.length > 0)) {  
      // there are some IR points to work with
      Point firstPt = new Point( irPts[0].getX(), irPts[0].getY());
      Point secondPt = new Point(0,0);
      
      // find a second point that is not close to the first
      boolean foundPair = false;
      int i=1;
      while (i < irPts.length) {
        secondPt.setLocation(irPts[i].getX(), irPts[i].getY());
        if (!isClose(firstPt, secondPt)) {
          foundPair = true;
          break;
        }
        i++;
      }

      if (foundPair) {    // got two different IR points
        if (firstPt.x > secondPt.x) {
          // swap so in left-right order
          Point temp =  firstPt;
          firstPt = secondPt;
          secondPt = temp;
        }

        // store the two points in the leftIR and rightIR globals
        leftIR.setLocation(firstPt);
        rightIR.setLocation(secondPt);

        // calculate width between points
        widthIR =  (int) Math.round(leftIR.distance(rightIR));    
      }
      else  {  /* did not find a different second point, so try guessing 
                  based on one of the leftIR/rightIR points calculated 
                  earlier. Use the movement of the first point relative to
                  the IR point to update the other point. */
        if (isClose(firstPt, leftIR)) {    // guess based on leftIR
          int xOffset = firstPt.x - leftIR.x;    // calculate offset of firstPt
          int yOffset = firstPt.y - leftIR.y;
          leftIR.setLocation(firstPt);
          rightIR.translate(xOffset, yOffset);   // apply offset to old rightPt
          widthIR =  (int) Math.round(leftIR.distance(rightIR));    
        }
        else if (isClose(firstPt, rightIR)) {  // guess based on rightIR
          int xOffset = firstPt.x - rightIR.x;    // calculate offset of firstPt
          int yOffset = firstPt.y - rightIR.y;
          rightIR.setLocation(firstPt);
          leftIR.translate(xOffset, yOffset);   // apply offset to old leftPt
          widthIR =  (int) Math.round(leftIR.distance(rightIR));    
        }
      }
    }

    return averagePoints(leftIR, rightIR);
  }  // end of locateIR()



  private Point averagePoints(Point p0, Point p1)
  // average the two points, staying within the panel's dimensions
  {
    int x = (p0.x + p1.x)/2;
    if (x < 0)
      x = 0;
    else if (x > pWidth-1)
      x = pWidth-1;

    int y = (p0.y + p1.y)/2;
    if (y < 0)
      y = 0;
    else if (y > pHeight-1)
      y = pHeight-1;

    return new Point(x, y);
  }  // end of averagePoints()



  private boolean isClose(Point p0, Point p1)
  {  return (p0.distance(p1) < POINT_PROXIMITY);  }


  // ---------------------- motion sensing ---------------------------


  public void onMotionSensingEvent(MotionSensingEvent e)
  { 
    Orientation ori = e.getOrientation();
    if (ori != null)
      rollAngle = (int)Math.round(ori.getRoll());  // roll is around y-axis
    // System.out.println("rollAngle: " + rollAngle);
  }  // end of onMotionSensingEvent()




  // ----------------------- accessing wiimote input -------------------


  public int getRoll()
  // this method is available to the outer Swing application;
  // used by WiiPosPanel.update()
  {  return rollAngle;  }


  /* add more public methods as a way of allowing the outer Swing application
     to read data from the wiimote */



  // ---------------------- button event processing ---------------------------


  public void onButtonsEvent(WiimoteButtonsEvent e)
  /* The user employs the arrow keys to set the wiimote's view boundaries
     for the sensor bar. 

     A good time to set these  boundaries is when the 
     IR location dot is drawn in red in the panel, which indicates
     that no IR data was captured by the wiimote. This means that
     the wiimote has probably moved out of view range of the 
     sensor bar.

     Use the 'A' button to signal a desire to shutdown.
  */
  { if (e.isButtonLeftPressed())
      xMinView = currLoc.x;
    if (e.isButtonRightPressed())
      xMaxView = currLoc.x;
    if (e.isButtonUpPressed())
      yMinView = currLoc.y;
    if (e.isButtonDownPressed())
      yMaxView = currLoc.y;
    
    if (e.isButtonAPressed())   // set wantsToFinish, which is read by the outer Swing app
      wantsToFinish = true;
  }  // end of onButtonsEvent()



  // -------------------- termination methods -----------------------

  public boolean wantsToFinish()
  // called by the Swing app to detect if the wiimote has signalled to stop
  {  return wantsToFinish;  }


  public void closeDown()
  // called from the Swing app to close down the wiimote
  {
    System.out.println();
    System.out.println("==================== Closing Values =======================");

    System.out.println("IR current tracking location: (" + currLoc.x + ", " +
                                                           currLoc.y + ")");
    System.out.println("Line width: " + widthIR);

    System.out.println("Sensor Bar Views. x-range: " + xMinView + "--" + xMaxView + 
                      ", y-range: " + yMinView + "--" + yMaxView + "\n");

    if (wiimote != null)
      WiiUseApiManager.shutdown();
  }  // end of closeDown()





  // ---------------------- other wiimote event processing ---------------------------


  public void onStatusEvent(StatusEvent e) 
  { if (showStatus) {
      System.out.println("\n" + e);  // only show at start-up
      showStatus = false;
    }
  }


  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e) { }
  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e) { }

  public void onDisconnectionEvent(DisconnectionEvent e) { }

  public void onExpansionEvent(ExpansionEvent e) { }

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e) { }
  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e) { }

  public void onNunchukInsertedEvent(NunchukInsertedEvent e) { }
  public void onNunchukRemovedEvent(NunchukRemovedEvent e) { } 



  // -------------------- drawing -----------------------------------
  // called by the outer Swing app so the wiimote can draw things in the panel


  public void draw(Graphics2D g2d)
  /*  Draw IR tracking information onto the drawing panel. 
      This includes: 
        * the calculated current tracking location (in red or black)
             - red means that the position is being calculated from old data
               since no IR points were received from the wiimote

        * a rotated green line that shows the current roll angle of the
          wiimote. The length of the line is the distance between IR points,
          so represents the length of the sensor bar
              - the length of the line is written in black above the line

        * the IR points seen by the wiimote are drawn in gray. If no points
          were detected then the current tracking location point is colored red;

        * the viewing boundaries of the wiimote are drawn as 4 red lines -- two horizontal
          and 2 vertical, representing the range over which the wiimote can see 
          the sensor bar. These lines are set by the user pressing the wiimote arrow keys
          to set the boundaries. 
               - the 4 corners of these boundaries are linked by 2 dotted lines along
                 the diagonals 
  */ 
  {
    g2d.setStroke(new BasicStroke(3));
    g2d.setFont(msgsFont);

    // draw current IR tracking location
    if ((irPts == null) || (irPts.length == 0))
      g2d.setColor(Color.RED);    
      /* use a warning red color if the location is not based on any 
         IR points data. This is a good time to set the view ranges
         with the arrow keys. */
    else
      g2d.setColor(Color.BLACK); 
    g2d.fillOval(currLoc.x-DOT_SIZE/2, currLoc.y-DOT_SIZE/2, DOT_SIZE, DOT_SIZE);

    // print pixel width between the left and right IR points
    g2d.setColor(Color.BLACK); 
    g2d.drawString(""+widthIR, currLoc.x-20, currLoc.y-30);


    /* 'reflect' the rollAngle so drawing reflects the position of
       the Wiimote in the user's hand */
    double drawAngle = Math.toRadians(-rollAngle);

    /* draw line showing IR width, centered on the tracking
       location, and rotated using the wiimote roll. */
    g2d.setColor(Color.GREEN);
    int xLen2 = (int) Math.round(widthIR/2 * Math.cos(drawAngle));
    int yLen2 = (int) Math.round(widthIR/2 * Math.sin(drawAngle));
    g2d.drawLine(currLoc.x - xLen2, currLoc.y + yLen2,
                 currLoc.x + xLen2, currLoc.y - yLen2);

    // draw the IR points seen by the wiimote
    if (irPts != null) {
      g2d.setColor(Color.LIGHT_GRAY);
      for (IRSource pt : irPts)
        g2d.fillOval(pt.getX()-DOT_SIZE/2, pt.getY()-DOT_SIZE/2, 
                                              DOT_SIZE, DOT_SIZE);    
    }

    // draw view range lines
    g2d.setColor(Color.RED); 
    g2d.drawLine(xMinView, 0, xMinView, pHeight);
    g2d.drawLine(xMaxView, 0, xMaxView, pHeight);

    g2d.drawLine(0, yMinView, pWidth, yMinView);
    g2d.drawLine(0, yMaxView, pWidth, yMaxView);

    // draw dashed lines between corners of view range lines
    Stroke dashed = new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 
                                    0, new float[]{9}, 0);
    g2d.setStroke(dashed);
    g2d.drawLine(xMinView, yMinView, xMaxView, yMaxView);
    g2d.drawLine(xMinView, yMaxView, xMaxView, yMinView);
  }  // end of draw()


}  // end of WiiReader class
