
// WiiSimple.java
// Andrew Davison, June 2014, ad@fivedots.coe.psu.ac.th

/* A simple example of how to use WiiUseJ by implementing
   WiimoteListener. My other examples are versions of this
   one, with more of the listener methods implemented, and 
   more complex code.

   Connects to a wiimote, and reports button events only.
   The 'A' button causes a disconnect.
*/



import wiiusej.*;
import wiiusej.wiiusejevents.physicalevents.*;
import wiiusej.wiiusejevents.utils.WiimoteListener;
import wiiusej.wiiusejevents.wiiuseapievents.*;



public class WiiSimple implements WiimoteListener
{

  // attempt to load the wiiuse library (wiiuse.dll)
  static {
    try {
      System.loadLibrary("wiiuse");
    } 
    catch (UnsatisfiedLinkError e) {
      System.err.println("wiiuse library failed to load\n" + e);
      System.exit(1);
    }
  }

  private Wiimote wiimote;
  private boolean showStatus = true;
                // flag for showing status info only once
  private int eventCount = 0;
                // useful for labelling event print-outs

  public WiiSimple() 
  {
    // look for a connected wiimote
    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, false);  
               // "false" means that there is no rumble on connection
    if ((wiimotes == null) || (wiimotes.length == 0)){
      System.out.println("No wiimote found");
      return;
    }
    else 
      System.out.println("No. of wiimotes found: " + wiimotes.length);
    wiimote = wiimotes[0];
    wiimote.setTimeout((short)20, (short)20);     
                 // default of 10 causes packet timeouts

    // motion sensing and IR tracking must be explicitly activated
    // wiimote.activateMotionSensing();
    // wiimote.activateIRTRacking();

    wiimote.addWiiMoteEventListeners(this);
    wiimote.getStatus();    // trigger a status event
  }  // end of WiiSimple()


  // --------------- wiimote event processing --------------


  public void onButtonsEvent(WiimoteButtonsEvent e) 
  { // processes button events, such as...
    System.out.println("Button (" + (eventCount++) + "): " + e); 

    // say goodbye using "A" is pressed
    if (e.isButtonAPressed()) {
      WiiUseApiManager.shutdown();
      System.out.println("Wiimote shutdown");
      System.exit(1);
    }
  }  // end of onButtonsEvent()



  public void onStatusEvent(StatusEvent e) 
  { if (showStatus) {    // report the status only once
      System.out.println(e);
      showStatus = false;
    }
  }


  public void onDisconnectionEvent(DisconnectionEvent e)
  {  System.out.println("Disconnection): " + e);   
     // should be activated when wiimote is disconnected, but it isn't 
  }

  public void onMotionSensingEvent(MotionSensingEvent e){ }

  public void onIrEvent(IREvent e) {}


  public void onExpansionEvent(ExpansionEvent e){}  
     // triggered when an expansion devices generates an event

  // insertion and removal listeners for three expansion devices
  public void onClassicControllerInsertedEvent(ClassicControllerInsertedEvent e) {}
  public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent e) {}

  public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent e) {}
  public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent e) {}

  public void onNunchukInsertedEvent(NunchukInsertedEvent e) {}
  public void onNunchukRemovedEvent(NunchukRemovedEvent e) {}


  // ----------------------------------------

  public static void main(String[] args)
  {  new WiiSimple();  }  

}  // end of WiiSimple class

